function main()
clc
clear
disp('Start decoding.....')
n_dft=128;
fc=739e6; % Center frequency to search
n_rb=6; % No. of RBs
f_search_set=-35e3:5e3:35e3; % Frequency offsets to be searched
ds_comb_arm=2;
thresh1_n_nines=12;
thresh2_n_sigma=3.0;
rx_cutoff=(n_rb*12*15e3/2+4*15e3)/(fs_lte/16/2);

disp('Reading data.....')
%%%%%%%%%% Fetching sampled data %%%%%%%%%%%
Fetch_col = 10000 ; % No. of samples defined in LabView
Fetch_raw = 16 ; % This will determine the time duration for sampling

Samples_ip = csvread('LTE_inphase.csv'); % Reading Collected data file. I/Q data in one File.
Samples_qp = csvread('LTE_quadrature.csv'); % Reading Collected data file. I/Q data in one File.
Samples = Samples_ip+ 1i*Samples_qp;
Vector = reshape(transpose(Samples),1,[]); % Generating a vector from collected data for further procissing.
Vector = Vector(100000:end);
%%%%%%%%% Processing %%%%%%%%%%%%%%%

n_cap=307200*8/16;
capbuf=Vector(1:n_cap); % This much of samples extract from the data 
disp('Detecting PSS.....')
[xc_incoherent_collapsed_pow xc_incoherent_collapsed_frq n_comb_xc n_comb_sp xc_incoherent_single xc_incoherent sp_incoherent xc sp]=xcorr_pss(capbuf,f_search_set,ds_comb_arm,fc);

% Suppose R=abs(Q1)^2+abs(Q2)^2+...abs(QN)^2:
% Then, R is chi-squared distributed with 2N degrees of freedom.
R99=chi2inv(.99,2*n_comb_xc*(2*ds_comb_arm+1));
R999=chi2inv(.999,2*n_comb_xc*(2*ds_comb_arm+1));
R9999=chi2inv(.9999,2*n_comb_xc*(2*ds_comb_arm+1));
R99999=chi2inv(.99999,2*n_comb_xc*(2*ds_comb_arm+1));
R999999=chi2inv(.999999,2*n_comb_xc*(2*ds_comb_arm+1));
R_mean=2*n_comb_xc*(2*ds_comb_arm+1);
  % Suppose Z=(abs(Y1)^2+abs(Y2)^2+...abs(YN)^2)/N:
  % Z=R*(sp_incoherent/rx_cutoff/137/2)/N
Z99=R99*sp_incoherent/rx_cutoff/137/2/n_comb_xc/(2*ds_comb_arm+1);
Z999=R999*sp_incoherent/rx_cutoff/137/2/n_comb_xc/(2*ds_comb_arm+1);
Z9999=R9999*sp_incoherent/rx_cutoff/137/2/n_comb_xc/(2*ds_comb_arm+1);
Z99999=R99999*sp_incoherent/rx_cutoff/137/2/n_comb_xc/(2*ds_comb_arm+1);
Z999999=R999999*sp_incoherent/rx_cutoff/137/2/n_comb_xc/(2*ds_comb_arm+1);
Z_mean=R_mean*sp_incoherent/rx_cutoff/137/2/n_comb_xc/(2*ds_comb_arm+1);

% Find the candidate peaks
R_th1=chi2inv(1-10^-thresh1_n_nines,2*n_comb_xc*(2*ds_comb_arm+1));
Z_th1=R_th1*sp_incoherent/rx_cutoff/137/2/n_comb_xc/(2*ds_comb_arm+1);
peaks=peak_search(xc_incoherent_collapsed_pow,xc_incoherent_collapsed_frq,Z_th1,f_search_set);  
figure(1);
plot( 1:9600,db10([transpose(xc_incoherent_collapsed_pow)]));
xlabel('OFDM Samples');
xlim([0 9600]);
zgo;

% Search through the detected PSS peaks  
%disp('full peak display disabled...');
disp('Detecting SSS....')
for pk=1:1
    peak_out=sss_detect(peaks(pk),capbuf,thresh2_n_sigma,fc);
    peaks(pk)=peak_out;
    if (isnan(peaks(pk).n_id_1))
      disp('Thresh2 fail');
      log_thresh2_fail(nt)=1;
      continue;
    end
   
    peak_out=pss_sss_foe(peaks(pk),capbuf,fc); % Calculating freq. offset
    peaks(pk)=peak_out;

    disp(sprintf('Rx. Power Level : %.2f dB',db10(peaks(pk).pow)));
    disp(sprintf('SSS/PSS : %i/%i',peaks(pk).n_id_1,peaks(pk).n_id_2));
    disp(sprintf('PCI : %i',3*peaks(pk).n_id_1+peaks(pk).n_id_2));
    disp(['Cyclic Prefix : ' peaks(pk).cp_type]);
    disp(sprintf('Start of the frame : %i',peaks(pk).frame_start));
    disp(sprintf('Identified Freq. Offset : %.3f Hz',peaks(pk).freq_fine));

    % Create the time/frequency grid
    [tfg tfg_timestamp]=extract_tfg(peaks(pk),capbuf,fc); % tfg : Time, Freq. grid

    % Perform FOE-C and TOE-C
    [tfg_comp tfg_comp_timestamp peaks_out]=tfoec(peaks(pk),tfg,tfg_timestamp,fc); % Freq. offset error correction and timing offset error correction
    peaks(pk)=peaks_out;

    disp('Detecting MIB.....')
    % Decode the MIB
    peaks_out=decode_mib(peaks(pk),tfg_comp);
    peaks(pk)=peaks_out;
    disp(sprintf('No. of RBs : %i',peaks(pk).n_rb_dl)); % No. of RBs
    disp(sprintf('System frame no. (SFN): %i',peaks(pk).sfn)); % SFN
    disp(sprintf('Antenna Ports : %i',peaks(pk).n_ports)); % No. of Antenna ports
    %RSRP calculation%
    [rsrp_pow_dbm_0,rsrp_timestamp_0]=rsrp_pow_cal(peaks(pk),tfg_comp,tfg_comp_timestamp,0);
    [rsrp_pow_dbm_1,rsrp_timestamp_1]=rsrp_pow_cal(peaks(pk),tfg_comp,tfg_comp_timestamp,1);
    [rsrp_pow_dbm_2,rsrp_timestamp_2]=rsrp_pow_cal(peaks(pk),tfg_comp,tfg_comp_timestamp,2);
    [rsrp_pow_dbm_3,rsrp_timestamp_3]=rsrp_pow_cal(peaks(pk),tfg_comp,tfg_comp_timestamp,3);
    figure(3);
    subplot(4,1,1);
    plot(rsrp_timestamp_0,rsrp_pow_dbm_0);
    subplot(4,1,2);
    plot(rsrp_timestamp_1,rsrp_pow_dbm_1);
    subplot(4,1,3);
    plot(rsrp_timestamp_2,rsrp_pow_dbm_2);
    subplot(4,1,4);
    plot(rsrp_timestamp_3,rsrp_pow_dbm_3);
end
% nt=1;
% log_success(nt)=0;
%   log_success_mib(nt)=0;
%   for pk=1:length(peaks)
%     if ( ...
%       (peaks(pk).n_id_1==92)&& ...
%       (peaks(pk).n_id_2==1)&& ...
%       (abs(peaks(pk).frame_start-17449.5250338295)<4.5)&& ...
%       strcmpi(peaks(pk).cp_type,'normal') ...
%     )
%       peaks(pk)
%       log_success(nt)=1;
%       log_foe(nt)=peaks(pk).freq_fine;
%       if ( ...
%         (peaks(pk).n_rb_dl==50)&& ...
%         (peaks(pk).sfn==649) ...
%       )
%         log_success_mib(nt)=1;
%       end
%       break;
%     end
%   end
%   %sum(log_success(1:nt)/nt)
% sum(log_success_mib(1:nt)/nt)
% state=etc(state,nt/n_trials);

  